# Generated by Django 4.0.10 on 2023-06-30 11:20

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aadiscordbot', '0010_alter_discordbot_options_ticketgroups'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='ticketgroups',
            options={'default_permissions': (), 'verbose_name': 'Ticket Cog Configuration',
                     'verbose_name_plural': 'Ticket Cog Configuration'},
        ),
        migrations.AddField(
            model_name='ticketgroups',
            name='ticket_channel',
            field=models.ForeignKey(
                default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='aadiscordbot.channels'),
        ),
    ]
