# Generated by Django 3.2.8 on 2022-01-04 08:04

from django.db import migrations, models


def add_settings(apps, schema_editor):
    AuthBotConfiguration = apps.get_model(
        'aadiscordbot', 'AuthBotConfiguration')

    # StructureLostShields
    AuthBotConfiguration.objects.create()


class Migration(migrations.Migration):

    dependencies = [
        ('aadiscordbot', '0003_authbotconfiguration'),
    ]

    operations = [
        migrations.RunPython(add_settings)
    ]
